#include "config.h"
#include "include.h"
#include "data.h"
#include "socket_msg.h"
#include "socket_handler.h"

static int handle_client(struct socket_msg *msg, int response) {
	struct wifi_client *own_client;

	own_client = avl_find_element(&clients_by_addr, msg->client->addr, own_client, avl);
	if (own_client) {
		// know client with higher freq
		if (own_client->freq_highest < msg->client->freq_highest) {
			return SOCKET_HANDLER_RESPONSE;
		}
		// learn freq
		if (own_client->freq_highest < msg->client->freq_highest) {
			own_client->freq_highest = msg->client->freq_highest;
		}
		if (!own_client->authed) {
			own_client->authed = msg->client->authed;
		}
		if (own_client->time < msg->client->time) {
			own_client->time = msg->client->time;
		}
		return response;
	}
	if(!config_client_socket_learning)
		return SOCKET_HANDLER_SILENCE;
	// learn client
	msg->client->try_probe = 0;
	msg->client->try_auth = 0;
	msg->client->connected = false;

	msg->client->avl.key = msg->client->addr;
	avl_insert(&clients_by_addr, &msg->client->avl);
	return SOCKET_HANDLER_SILENCE;
}

int socket_handler(struct socket_msg *msg) {
	if(msg->type & SOCKET_MSG_TYPE_RESPONSE) {
		return SOCKET_HANDLER_ERROR;
	}
	int ret = SOCKET_HANDLER_RESPONSE;
	if(msg->type & SOCKET_MSG_TYPE_REQUEST) {
		msg->type = (msg->type ^ SOCKET_MSG_TYPE_REQUEST) | SOCKET_MSG_TYPE_RESPONSE;

		// single handlers
		if(msg->type & SOCKET_MSG_TYPE_CLIENT) ret = handle_client(msg, SOCKET_HANDLER_RESPONSE);

		return ret;
	}

	// single handlers
	if(msg->type & SOCKET_MSG_TYPE_CLIENT) ret = handle_client(msg, SOCKET_HANDLER_SILENCE);

	return ret;

}
